#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "folgeschrittberechnung.h"
#include "kollision.h"
#include "ausgabe.h"
#include "bspzustaende.h"
#include "integration.h"
#include "kollisionsbehandlung.h"
#include "bewegungsgleichungen.h"
#include "io.h"
#include "gleichungsloeser.h"
#include "beruehrung.h"
#include "stoss.h"


#define REALFORMAT     "%lf " 
#define TIMEFORMAT     "%lf "

#define MAXLISTSIZE         80

enum formats {STATE, BODIES, BODYLIST, LIST};

static int outputformat = STATE;
static int listsize = 0;
static TReal  *reallist[MAXLISTSIZE];
static TKoerper *bodylist[MAXLISTSIZE];
static TZustand *z;

static TReal sim_time = 3.0;
static TReal output_rate = 0.01;
static TKonfigFBB konfig;

static TBoolean show_istat = FALSE;
static TBoolean zungeaendert = TRUE;
static TBoolean collision = FALSE;
static TMaterialTabelle *mat_tab = NULL;


/*
Argumente von fsb:


(config <pendel> <...>)

simtime <real>
istep <real> 
ostep <real>
state
bodies
body <bodylist>
list <varlist>

<bodylist> = Zahlenliste 1,2,3...
<varlist> =

t               Zeit 
1px 1py 1pz     Position
1vx 1vy 1vz     Geschwindigkeit
1ax 1ay 1az     Beschleunigung

1wx 1wy 1wz     Winkelgeschwindigkeit
1ux 1uy 1uz     Winkelbeschleunigung
1q0 1q1 1q2 1q3 Eulerparameter

*/



void step_output()
{
    int i;

    switch (outputformat) {
      case STATE:
	printzustand(z);
	break;
      case BODIES:
	printf(">>> t=");
	printf(TIMEFORMAT, z->Zeit);
	printf("\n");
	printkoeliste(z->Koerper);
	break;
      case BODYLIST:
	for (i=0; i<listsize; i++)
	  printkoerper(bodylist[i]);
	break;
      case LIST:
	for (i=0; i<listsize; i++)
	  printf(REALFORMAT, *reallist[i]);
	printf("\n");
	break;
      default:
	printf("*** wrong output format\n");
	exit(10);
	break;
    }
}



void scanreal(int argc, char *argv[], int *i, TReal *pr)
{
    TReal h;

    if ((*i) >= argc) {
	printf("*** real-number expected after %s\n\n", argv[*i]);
	exit(15);
    }
    else {
	h = *pr;
	if (sscanf(argv[*i], REALFORMAT, pr) == 0) {
	    printf("*** wrong real-parameter %s\n\n", argv[*i]);
	    *pr = h;
	    exit(14);
	}
	else (*i)++;
      }
}    

void scanint_d(int argc, char *argv[], int *i, int *pi, int def)
{
    if ((*i) >= argc) {
	*pi = def;
    }
    else {
	if (sscanf(argv[*i], "%d", pi) == 0) {
	    *pi = def;
	}
	else (*i)++;
      }
}    




TKoerper *getk(int nr)
{
    int n;
    TKoerper *k;

    k = z->Koerper;
    n=0;
    while (k) {
	n++;
	if (nr == n) break;
	k = k->Naechster;
    }
 
   if (nr == n) {
       return k;
    }
    else {
	printf("*** no body with nr. %d exists, max=%d\n\n", nr, n);
	exit(12);
    }
}

void addlist(TReal *pr)
{
    if (listsize < MAXLISTSIZE) {
	reallist[listsize] = pr;
	listsize++;
	outputformat = LIST;
    }
    else {
	printf("*** too man vars in list, max is %d\n\n", MAXLISTSIZE);
	exit(13);
    }
}


void scanbodylist(int argc, char *argv[], int *i)
{
    int nr;
    
    listsize = 0;
    while ((*i) < argc) {
	if (sscanf(argv[*i], "%d", &nr) == 0) break;
	else {
	    if (listsize < MAXLISTSIZE) {
		bodylist[listsize] = getk(nr);
		listsize++;
		outputformat = BODYLIST;
	    }
            else {
		printf("*** too man bodies in list, max is %d\n\n", MAXLISTSIZE);
		exit(13);
	    }
	    (*i)++;
	}
    }
}



void scanlist(int argc, char *argv[], int *i)
{
    TKoerper *k;
    int nr;
    char v, e;
    int j;


    listsize = 0;
    while ((*i) < argc) {
	
	switch(argv[*i][0]) {
	  case 't':
	    addlist(&(z->Zeit));
	    break;

	  case '0':  case '1':  case '2':  case '3':  case '4':
	  case '5':  case '6':  case '7':  case '8':  case '9':  
	    if (sscanf(argv[*i],"%d%c%c", &nr, &v, &e) != 3) {
		printf("*** incorrect real-var %s\n\n", argv[*i]);
		exit(16);
	    }

	    k = getk(nr);

            switch(e) {
	      case '0': case 'x':
		j = 0;
		break;
	      case '1': case 'y':
		j = 1;
		break;
	      case '2': case 'z':
		j = 2;
		break;
	      default:

		if (v != 'q') printf("*** wrong element-selection %c in %s\n\n", e, argv[*i]);
		break;
	    }

	    switch(v) {
	      case 'p':
		addlist(&(k->Position[j]));
		break;
	      case 'v':
		addlist(&(k->Geschwindigkeit[j]));
		break;
	      case 'a':
		addlist(&(k->Beschleunigung[j]));
		break;
	      case 'w':
		addlist(&(k->Drehgeschwindigkeit[j]));
		break;
	      case 'u':
		addlist(&(k->Drehbeschleunigung[j]));
		break;
	      case 'q':
		switch(e) {
		  case '0': addlist(&(k->q[0])); break;
		  case '1': addlist(&(k->q[1])); break;
		  case '2': addlist(&(k->q[2])); break;
		  case '3': addlist(&(k->q[3])); break;
		  default:
		    printf("*** unknow eulerpar %s, eulerpars only q0 .. q3\n\n", argv[*i]); 
		    break;
		}
		break;
	      default:
		printf("*** unknown vektor-var %s\n\n", argv[*i]);
		exit(17);
		break;
	    }
	    break;
	    
	  default:
	    return ;
	}
	(*i)++;
    }
}



void parseargs(int argc, char *argv[])
{
    int i = 1;
    char *s;

    while (i<argc) {
	for (s=argv[i]; *s; s++)
	  *s = tolower(*s);

	s = argv[i];

	if (strcmp(s, "world") == 0) {
	    int j = 0;

	    i++;
	    if (i<argc)
	      for (j=0; worldnames[j]; j++) {
		  if (strcmp(argv[i], worldnames[j]) == NULL) {
		      z = worlds[j]; 
		      i++;
		      break;
		  }
	      }

	    if (worldnames[j] == NULL || i>argc) {
		if (i<argc)
		  printf("*** world name %s unknown!\n", argv[i]);
		else 
		  printf("*** world name expected\n");

		printf("\navailable worlds: ");
		for (j=0; worldnames[j]; j++)
		  printf("%s%s", (j==0) ? " " : ", ", worldnames[j]);
		printf("\n\n");
		exit(20);
	    }

	    InitFBB(z);
	}
	else if (strcmp(s, "istat") == 0) {
	    i++;
	    show_istat = TRUE;
	}

	/* Debug-Parameter
	 */

	else if (strcmp(s, "debug_b") == 0) {
	    i++;
	    scanint_d(argc, argv, &i, &b_debug_level, 1);
	}
	else if (strcmp(s, "debug_gl") == 0) {
	    i++;
	    scanint_d(argc, argv, &i, &gl_debug_level, 1);
	}
	else if (strcmp(s, "debug_i") == 0) {
	    i++;
	    scanint_d(argc, argv, &i, &i_debug_level, 1);
	}
	else if (strcmp(s, "debug_kb") == 0) {
	    i++;
	    scanint_d(argc, argv, &i, &kb_debug_level, 1);
	}
	else if (strcmp(s, "debug_s") == 0) {
	    i++;
	    scanint_d(argc, argv, &i, &s_debug_level, 1);
	}
	else if (strcmp(s, "debug_ke") == 0) {
	    i++;
	    scanint_d(argc, argv, &i, &ke_debug_level, 1);
	}
	else if (strcmp(s, "debug_fsb") == 0) {
	    i++;
	    scanint_d(argc, argv, &i, &fsb_debug_level, 1);
	}
	else if (strcmp(s, "debug_bg") == 0) {
	    i++;
	    scanint_d(argc, argv, &i, &bg_debug_level, 1);
	}
	else if (strcmp(s, "debug_kr") == 0) {
	    i++;
	    scanint_d(argc, argv, &i, &kr_debug_level, 1);
	}


	else if (strcmp(s, "load") == 0) {
	    i++;
	    if (ReadZustand(&z, mat_tab, argv[i], NULL)==FALSE) {
		printf("*** Fehler beim Lesen des Zustandes!!\n\n");
		exit(22);
	    }
	    i++;
	}
	else if (strcmp(s, "save") == 0) {
	    i++;
	    if (WriteZustand(z, argv[i], NULL)==FALSE) {
		printf("*** Fehler beim Schreiben des Zustandes!!\n\n");
		exit(33);
	    }
	    i++;
	}

	else if (strcmp(s, "loadmat") == 0) {
	    i++;
	    if (ReadMaterialien(&mat_tab, argv[i], NULL)==FALSE) {
		printf("*** Fehler beim Lesen der Materialtabelle!!\n\n");
		exit(22);
	    }
	    z->MaterialDaten = mat_tab;
	    i++;
	}
	else if (strcmp(s, "savemat") == 0) {
	    i++;
	    if (WriteMaterialien(z->MaterialDaten, argv[i], NULL)==FALSE) {
		printf("*** Fehler beim Schreiben der Materialtabelle!!\n\n");
		exit(33);
	    }
	    i++;
	}

	else if (strcmp(s, "loadcon") == 0) {
	    i++;
	    if (ReadKonfiguration(&konfig, argv[i], NULL)==FALSE) {
		printf("*** Fehler beim Lesen der FBB-Konfiguration!!\n\n");
		exit(22);
	    }
	    KonfigFBB(&konfig);
	    i++;
	}
	else if (strcmp(s, "savecon") == 0) {
	    i++;
	    if (WriteKonfiguration(&konfig, argv[i], NULL)==FALSE) {
		printf("*** Fehler beim Schreiben der FBB-Konfiguration!!\n\n");
		exit(33);
	    }
	    i++;
	}


	else if (strcmp(s, "airres") == 0) {
	    i++;
	    konfig.L_Luftwiderstand = TRUE; 
	}
	else if (strcmp(s, "springimpact") == 0) {
	    i++;
	    konfig.K_FederStoss = TRUE;
	}
	else if (strcmp(s, "no_springimpact") == 0) {
	    i++;
	    konfig.K_FederStoss = FALSE;
	}
	else if (strcmp(s, "collision") == 0) {
	    i++;
	    collision = TRUE;
	}
	else if (strcmp(s, "xchange") == 0) {
	    i++;
	    zungeaendert = FALSE;
	}
	else if (strcmp(s, "penality") == 0) {
	    i++;
	    konfig.B_PenalityMethode = TRUE;
	}
	else if (strcmp(s, "stepcontrol") == 0) {
	    i++;
	    konfig.I_Fehlersteuerung = TRUE;
	}
	else if (strcmp(s, "conststep") == 0) {
	    i++;
	    konfig.I_Fehlersteuerung = FALSE;
	}
	else if (strcmp(s, "simtime") == 0) {
	    i++;
	    scanreal(argc, argv, &i, &sim_time);
	}
	else if (strcmp(s, "maxcv") == 0) {
	    i++;
	    scanreal(argc, argv, &i, &konfig.B_MaxBerGeschw);
	}
	else if (strcmp(s, "eps") == 0) {
	    i++;
	    scanreal(argc, argv, &i, &konfig.I_Eps);
	}
	else if (strcmp(s, "min_dh") == 0) {
	    i++;
	    scanreal(argc, argv, &i, &konfig.I_MinSchritt);
	}
	else if (strcmp(s, "max_dh") == 0) {
	    i++;
	    scanreal(argc, argv, &i, &konfig.I_MaxSchritt);
	}
	else if (strcmp(s, "istep") == 0) {
	    i++;
	    scanreal(argc, argv, &i, &konfig.I_Schritt);
	}
	else if (strcmp(s, "cstep") == 0) {
	    i++;
	    scanreal(argc, argv, &i, &konfig.K_KollSchritt);
	}
	else if (strcmp(s, "ostep") == 0) {
	    i++;
	    scanreal(argc, argv, &i, &output_rate);
	}
	else if (strcmp(s, "state") == 0) {
	    outputformat = STATE;
	    i++;
	}
	else if (strcmp(s, "bodies") == 0) {
	    outputformat = BODIES;
	    i++;
	}
	else if (strcmp(s, "body") == 0) {
	    i++;
	    scanbodylist(argc, argv, &i);
	}
	else if (strcmp(s, "list") == 0) {
	    i++;
	    scanlist(argc, argv, &i);
	}
	else {
	    if (*s != 'h' && strcmp(s, "help")) printf("*** unknown keyword %s\n\n", s); 

	    /* Help-Text !!!!!!!!! */
	    printf(
"Usage: %s [help] [simtime <time:real>] [istep <intstepsize:real>]\n"
"           [ostep <outputrate:real>] [eps <real>]\n"
"           [state | bodies | body <list> | list <varlist>]\n"
"           [world <name:string>]\n"
"           [istat]\n"
"           [debug_(b|gl|i|kb|s|ke|fsb|bg|kr) <level:int>]\n"
"           [load <statefile>] [save <filename>]\n"
"           [loadcon <configfile>] [savecon <filename>]\n"
"           [loadmat <materialfile>] [savemat <filename>]\n"
"           [airres] [springimpact] [no_springimpact] [collision]\n"
"           [xchange] [penality] [stepcontrol] [conststep]\n"
"           [maxcv <maxtouchvelocity:real>]\n"
"           [min_dh <minstepsize:real>] [max_dh <maxstepsize:real>]\n"
"           [cstep <collisionstepsize:real>]\n"
"  Note: <varlist> = {<bodynr><<p|v|a|w|u><x|y|z>|q<0|1|2|3>>}+\n"
"\n  p.e. %s simtime 10.0 istep 0.001 ostep 0.1 list t 2py 2pz 2wx 1vx\n"
"  default options settings:\n"
"    world %s, simtime %lf, ostep %lf\n"
"    istep %lf, eps 0.000010, state, conststep, no_springimpact\n\n",
		   argv[0], argv[0], worldnames[0], sim_time, output_rate, konfig.I_Schritt, konfig.I_Eps);

	    exit(11);  
	}
    } 
}



void main(int argc, char *argv[])
{
    TKollision *kol = NULL;


    ErfrageKonfigFBB(&konfig);
    konfig.I_Fehlersteuerung = FALSE;		/* "geheimer" Schalter in Integration */
    konfig.B_PenalityMethode = FALSE;
/*    konfig.K_SimStoss = FALSE; Veraltet */
    konfig.G_Zufall = FALSE;
    konfig.L_Luftwiderstand = FALSE; 


    InitFBB(z);

    z = worlds[0];
    parseargs(argc, argv);

    KonfigFBB(&konfig);

    if (collision==FALSE) {
	while (z->Zeit+output_rate <= sim_time) {
	    step_output();
	    FolgeBildBerechnung(z, output_rate);
	    if (zungeaendert) UngeaendertFBB(z);
	}
	step_output();
    }
    else {
	KoerperRotMatrizen(z->Koerper);
	KollisionsErkennung(z, &kol);
	KolListeLoeschen(&kol);
    }
    EndFBB(z);

    if (show_istat) {
	TIStatistik istat;
	
	IStatistik(&istat);
	printf("\n --- Statistik-Integrationsverfahren --- \n");
	printf("Fehlversuche wegen Ungenauigkeit : %ld\n", istat.AnzFehlversuche);
	printf("Intergrationsschritte            : %ld\n", istat.AnzSchritte);
	printf("Interpolationsaufrufe            : %ld\n", istat.AnzInterpolAufrufe);
	printf("Funktionsauswertungen            : %ld\n\n", istat.AnzFunkAuswertungen);
    }
}

