/********************************************************************************
 *  Projektname		: AERO
 *  Filename		: materialien.c
 *  Filetyp		: C-Source
 ********************************************************************************
 *  Modulname		: materialien.o
 *  letzte Aenderung	: 18.05.93
 *  Autor		: Horst Stolz (HUS), verkleinert von H. Keller
 *  Status:		: erfolgreich compiliert
 *  
 *  Beschreibung:
 *    liefert eine einfache Materialtabelle, von Horsts Routine sind nur die
 *    fuer mini wichtigen Routinen uebrig.
 *
 *  Noch zu mache:
 *    - mehr Materialien
 *    - Reibungs- und Elastizitaetskoeffizienten
 *
 *  Versionsgeschichte:
 *  -------------------
 *  03.12.92   File erstellt
 *  25.01.93   Mit "unrichtige" Elastizitaetswerte belegt
 *  18.05.93   Default-Werte fuer Raytracer eingefuegt
 ********************************************************************************/

#include <stdlib.h>

#include "materialien.h"


static char *MatNamen[33] = {
    "iron", "plastic", "cork", "ice", "glass", "granite", "gold", "alu", "wood",
    "concrete", "brick", "rubber", NULL
};

static TReal MatDichte[32] = {
    7.8e3, 0.9e3, 0.27e3, 0.917e3, 2.5e3, 2.8e3,
    19.29e3, 2.7e3, 0.6e3, 2.3e3, 1.5e3, 0.94e3
};

static TReal MatGleitfaktor[32] = {
	1.0, 0.4, 0.5, 0.014, 0.02, 0.6, 0.9, 0.98, 0.5, 0.2, 1.2, 3.5    
};

static TReal MatHaftfaktor[32] = {
    1.3, 0.5, 0.6, 0.002, 0.04, 0.8, 0.95, 1.05, 0.65, 0.3, 1.5, 4.0
};

static TReal MatStossfaktor[32] = {
    0.9, 0.95, 0.8, 0.8, 0.97, 0.3, 0.2, 0.92, 0.7, 0.5, 0.4, 0.99
};

static TReal MatDehnfaktor[32] = {
    0.5e5, 0.5e5, 0.5e5, 0.5e5, 0.5e5, 0.5e5, 0.5e5, 0.5e5, 0.5e5, 0.5e5, 0.5e5, 0.5e5

};

static TReal MatDaempffaktor[32] = {
    0.5e3, 0.5e3, 0.5e3, 0.5e3, 0.5e3, 0.5e3, 0.5e3, 0.5e3, 0.5e3, 0.5e3, 0.5e3, 0.5e3
};

static unsigned short MatDefaultR[32] = {
    23550, 16383, 37887, 52735, 65535, 26111, 65535, 56319, 38911, 32767, 59391, 48639
};

static unsigned short MatDefaultG[32] = {
    27135, 50687, 29695, 63487, 65535, 24063, 55295, 55295, 24574, 34815, 19967, 44031
};

static unsigned short MatDefaultB[32] = {
    36862, 11263, 20991, 65535, 65535, 21503, 0, 56831, 0, 37375, 12287, 36351
};

static TReal MatDurchsichtigkeit[32] = {
    0.0, 0.0, 0.0, 0.6, 0.9, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0
};

static TReal MatReflexion[32] = {
    0.1, 0.0, 0.0, 0.3, 0.7, 0.0, 0.4, 0.1, 0.0, 0.0, 0.0, 0.0
};

static TReal MatRauhigkeit[32] = {
    0.0, 0.03, 0.1, 0.0, 0.0, 0.1, 0.0, 0.03, 0.05, 0.08, 0.07, 0.02 
};

static unsigned short MatFlags[32];


TMaterialTabelle GrundMaterialien = {
    12,
    32,
    MatNamen,
    MatHaftfaktor,
    MatGleitfaktor, 
    MatStossfaktor,
    MatDichte,
    MatDehnfaktor,
    MatDaempffaktor,
    MatDefaultR,
    MatDefaultG,
    MatDefaultB,
    MatDurchsichtigkeit,
    MatReflexion,
    MatRauhigkeit,
    MatFlags
};




char **MaterialNamen(TMaterialTabelle *mt)
{
    return (mt->MaterialNamen);
}


int AnzahlMaterialien(TMaterialTabelle *mt)
{
    return mt->AnzahlMaterialien;
}


